"use strict";
/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.DrawPass = void 0;
var tslib_1 = require("tslib");
var util_1 = require("../../mol-gl/compute/util");
var schema_1 = require("../../mol-gl/renderable/schema");
var renderable_1 = require("../../mol-gl/renderable");
var shader_code_1 = require("../../mol-gl/shader-code");
var render_item_1 = require("../../mol-gl/webgl/render-item");
var mol_util_1 = require("../../mol-util");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var quad_vert_1 = tslib_1.__importDefault(require("../../mol-gl/shader/quad.vert"));
var depth_merge_frag_1 = tslib_1.__importDefault(require("../../mol-gl/shader/depth-merge.frag"));
var stereo_1 = require("../camera/stereo");
var DepthMergeSchema = tslib_1.__assign(tslib_1.__assign({}, util_1.QuadSchema), { tDepthPrimitives: schema_1.TextureSpec('texture', 'depth', 'ushort', 'nearest'), tDepthVolumes: schema_1.TextureSpec('texture', 'depth', 'ushort', 'nearest'), uTexSize: schema_1.UniformSpec('v2'), dPackedDepth: schema_1.DefineSpec('boolean') });
var DepthMergeShaderCode = shader_code_1.ShaderCode('depth-merge', quad_vert_1.default, depth_merge_frag_1.default);
function getDepthMergeRenderable(ctx, depthTexturePrimitives, depthTextureVolumes, packedDepth) {
    var values = tslib_1.__assign(tslib_1.__assign({}, util_1.QuadValues), { tDepthPrimitives: mol_util_1.ValueCell.create(depthTexturePrimitives), tDepthVolumes: mol_util_1.ValueCell.create(depthTextureVolumes), uTexSize: mol_util_1.ValueCell.create(linear_algebra_1.Vec2.create(depthTexturePrimitives.getWidth(), depthTexturePrimitives.getHeight())), dPackedDepth: mol_util_1.ValueCell.create(packedDepth) });
    var schema = tslib_1.__assign({}, DepthMergeSchema);
    var renderItem = render_item_1.createComputeRenderItem(ctx, 'triangles', DepthMergeShaderCode, schema, values);
    return renderable_1.createComputeRenderable(renderItem, values);
}
var DrawPass = /** @class */ (function () {
    function DrawPass(webgl, width, height) {
        this.webgl = webgl;
        var extensions = webgl.extensions, resources = webgl.resources;
        this.colorTarget = webgl.createRenderTarget(width, height);
        this.packedDepth = !extensions.depthTexture;
        this.depthTarget = webgl.createRenderTarget(width, height);
        this.depthTexture = this.depthTarget.texture;
        this.depthTargetPrimitives = this.packedDepth ? webgl.createRenderTarget(width, height) : null;
        this.depthTargetVolumes = this.packedDepth ? webgl.createRenderTarget(width, height) : null;
        this.depthTexturePrimitives = this.depthTargetPrimitives ? this.depthTargetPrimitives.texture : resources.texture('image-depth', 'depth', 'ushort', 'nearest');
        this.depthTextureVolumes = this.depthTargetVolumes ? this.depthTargetVolumes.texture : resources.texture('image-depth', 'depth', 'ushort', 'nearest');
        if (!this.packedDepth) {
            this.depthTexturePrimitives.define(width, height);
            this.depthTextureVolumes.define(width, height);
        }
        this.depthMerge = getDepthMergeRenderable(webgl, this.depthTexturePrimitives, this.depthTextureVolumes, this.packedDepth);
    }
    DrawPass.prototype.setSize = function (width, height) {
        var w = this.colorTarget.getWidth();
        var h = this.colorTarget.getHeight();
        if (width !== w || height !== h) {
            this.colorTarget.setSize(width, height);
            this.depthTarget.setSize(width, height);
            if (this.depthTargetPrimitives) {
                this.depthTargetPrimitives.setSize(width, height);
            }
            else {
                this.depthTexturePrimitives.define(width, height);
            }
            if (this.depthTargetVolumes) {
                this.depthTargetVolumes.setSize(width, height);
            }
            else {
                this.depthTextureVolumes.define(width, height);
            }
            mol_util_1.ValueCell.update(this.depthMerge.values.uTexSize, linear_algebra_1.Vec2.set(this.depthMerge.values.uTexSize.ref.value, width, height));
        }
    };
    DrawPass.prototype._render = function (renderer, camera, scene, helper, toDrawingBuffer, transparentBackground) {
        var _a = camera.viewport, x = _a.x, y = _a.y, width = _a.width, height = _a.height;
        renderer.setViewport(x, y, width, height);
        if (toDrawingBuffer) {
            this.webgl.unbindFramebuffer();
        }
        else {
            this.colorTarget.bind();
            if (!this.packedDepth) {
                this.depthTexturePrimitives.attachFramebuffer(this.colorTarget.framebuffer, 'depth');
            }
        }
        renderer.render(scene.primitives, camera, 'color', true, transparentBackground, 1, null);
        // do a depth pass if not rendering to drawing buffer and
        // extensions.depthTexture is unsupported (i.e. depthTarget is set)
        if (!toDrawingBuffer && this.depthTargetPrimitives) {
            this.depthTargetPrimitives.bind();
            renderer.render(scene.primitives, camera, 'depth', true, transparentBackground, 1, null);
            this.colorTarget.bind();
        }
        // do direct-volume rendering
        if (!toDrawingBuffer) {
            if (!this.packedDepth) {
                this.depthTextureVolumes.attachFramebuffer(this.colorTarget.framebuffer, 'depth');
                this.webgl.state.depthMask(true);
                this.webgl.gl.viewport(x, y, width, height);
                this.webgl.gl.scissor(x, y, width, height);
                this.webgl.gl.clear(this.webgl.gl.DEPTH_BUFFER_BIT);
            }
            renderer.render(scene.volumes, camera, 'color', false, transparentBackground, 1, this.depthTexturePrimitives);
            // do volume depth pass if extensions.depthTexture is unsupported (i.e. depthTarget is set)
            if (this.depthTargetVolumes) {
                this.depthTargetVolumes.bind();
                renderer.render(scene.volumes, camera, 'depth', true, transparentBackground, 1, this.depthTexturePrimitives);
                this.colorTarget.bind();
            }
        }
        // merge depths from primitive and volume rendering
        if (!toDrawingBuffer) {
            this.depthMerge.update();
            this.depthTarget.bind();
            // this.webgl.state.disable(this.webgl.gl.SCISSOR_TEST);
            this.webgl.state.disable(this.webgl.gl.BLEND);
            this.webgl.state.disable(this.webgl.gl.DEPTH_TEST);
            this.webgl.state.disable(this.webgl.gl.CULL_FACE);
            this.webgl.state.depthMask(false);
            this.webgl.state.clearColor(1, 1, 1, 1);
            this.webgl.gl.viewport(x, y, width, height);
            this.webgl.gl.scissor(x, y, width, height);
            this.webgl.gl.clear(this.webgl.gl.COLOR_BUFFER_BIT);
            this.depthMerge.render();
            this.colorTarget.bind();
        }
        if (helper.debug.isEnabled) {
            helper.debug.syncVisibility();
            renderer.render(helper.debug.scene, camera, 'color', false, transparentBackground, 1, null);
        }
        if (helper.handle.isEnabled) {
            renderer.render(helper.handle.scene, camera, 'color', false, transparentBackground, 1, null);
        }
        if (helper.camera.isEnabled) {
            helper.camera.update(camera);
            renderer.render(helper.camera.scene, helper.camera.camera, 'color', false, transparentBackground, 1, null);
        }
    };
    DrawPass.prototype.render = function (renderer, camera, scene, helper, toDrawingBuffer, transparentBackground) {
        if (stereo_1.StereoCamera.is(camera)) {
            this._render(renderer, camera.left, scene, helper, toDrawingBuffer, transparentBackground);
            this._render(renderer, camera.right, scene, helper, toDrawingBuffer, transparentBackground);
        }
        else {
            this._render(renderer, camera, scene, helper, toDrawingBuffer, transparentBackground);
        }
    };
    return DrawPass;
}());
exports.DrawPass = DrawPass;
//# sourceMappingURL=draw.js.map